
#define A5SES_IIO_ADDON

#include <a5ses/Bitmap.hpp>
#include <a5ses/Display.hpp>
#include <a5ses/Event.hpp>
#include <a5ses/Keyboard.hpp>
#include <a5ses/Timer.hpp>

#include <memory>

using namespace a5;
using namespace std;

int main(int argc, const char *argv[])
{
	const char *filename;
	bool redraw = true;
	double zoom = 1;

	if (argc > 1)
	{
		filename = argv[1];
	}
	else
	{
		filename = "data/mysha.pcx";
	}
	
	Mouse mouse;
	Keyboard keyboard;
	
	Display display(640, 480, Display::Windowed);

	display.SetTitle(filename);
	
	Bitmap membitmap(filename);
	Bitmap bitmap(membitmap);

	Timer timer(1.f /30.f);
	
	Event_Queue queue;
	
	queue.Register(keyboard);
	queue.Register(display);
	queue.Register(timer);
	
	timer.Start();
	
	while (true)
	{
		auto_ptr<Event> e = queue.Wait();
		
		Display::Event *de;
		Keyboard::Event *ke;

		if (de = dynamic_cast<Display::Event *>(e.get()))
		{
			if (de->type == Display::Close)
			{
				break;
			}
		}
		else if (ke = dynamic_cast<Keyboard::Event *>(e.get()))
		{
			if (ke->keycode == ALLEGRO_KEY_ESCAPE)
			{
				break;
			}
			else if (ke->unichar == '1')
			{
				zoom = 1.0;
			}
			else if (ke->unichar == '+')
			{
				zoom *= 1.1;
			}
			else if (ke->unichar == '-')
			{
				zoom /= 1.1;
			}
			else if (ke->unichar == 'f')
			{
				zoom = display.Width() / bitmap.Width();
			}
		}
		else if (e->Type() == Timer::Event::TypeID)
		{
			auto_ptr<Timer::Event> te(dynamic_cast<Timer::Event *>(e.get()));
			
			if (te->source == &timer)
			{
				redraw = true;
			}
		}
		else if (redraw)
		{
			redraw = false;
			display.Clear();

			if (zoom == 1.0)
			{
				display.Blit(bitmap, 0, 0, 0, 0);
			}
			else
			{
				display.BlitScaled(bitmap 0, 0, Rectangle(), 0, 0);
			}

			display.Flip();
		/*
            redraw = false;
            al_clear_to_color(al_map_rgb_f(0, 0, 0));
            if (zoom == 1)
                al_draw_bitmap(bitmap, 0, 0, 0);
            else
                al_draw_rotated_scaled_bitmap(bitmap, 0, 0, 0, 0, zoom, zoom, 0, 0);
            al_flip_display();
        */
		}
	}
}
END_OF_MAIN()
